"use strict";
/*
 * MIT License
 *
 * Copyright (c) 2018 Choko (choko@curioswitch.org)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
const path_1 = __importDefault(require("path"));
const commander_1 = __importDefault(require("commander"));
const inquirer_1 = __importDefault(require("inquirer"));
const klaw_sync_1 = __importDefault(require("klaw-sync"));
const utils_1 = require("../utils");
const package_json_1 = __importDefault(require("../../../../package.json"));
let argDir;
commander_1.default
    .version(package_json_1.default.version)
    .arguments('[dir]')
    .action((dir) => (argDir = dir))
    .option('-n, --name <name>', 'Name of the package')
    .parse(process.argv);
function run() {
    return __awaiter(this, void 0, void 0, function* () {
        const packageJsonTemplate = yield fs_extra_1.readFile(path_1.default.join(__dirname, 'package.json.hbs'));
        const tsconfigTemplate = yield fs_extra_1.readFile(path_1.default.join(__dirname, 'tsconfig.json.hbs'));
        const baseTsConfigTemplate = yield fs_extra_1.readFile(path_1.default.join(__dirname, 'tsconfig.json'));
        const tslintConfigTemplate = yield fs_extra_1.readFile(path_1.default.join(__dirname, 'tslint.json.hbs'));
        const questions = [];
        let dir = argDir;
        if (!dir) {
            questions.push({
                name: 'dir',
                message: 'Enter the directory to create the package in',
            });
        }
        let name = commander_1.default.naame;
        if (!name) {
            questions.push({
                name: 'name',
                message: 'What is the name of the package?',
            });
        }
        if (questions.length > 0) {
            const answers = yield inquirer_1.default.prompt(questions);
            if (answers.dir) {
                dir = answers.dir;
            }
            if (answers.name) {
                name = answers.name;
            }
        }
        const packageDir = path_1.default.resolve(process.cwd(), dir);
        const context = {
            name,
            baseWebVersion: package_json_1.default.version,
            baseWebDevVersion: package_json_1.default.version,
            baseTsConfigLocation: path_1.default.join(path_1.default.relative(packageDir, process.cwd()), 'tsconfig.json'),
        };
        yield fs_extra_1.mkdirs(path_1.default.resolve(packageDir));
        yield utils_1.renderTemplate(packageJsonTemplate, path_1.default.resolve(packageDir, 'package.json'), context);
        yield utils_1.renderTemplate(tslintConfigTemplate, path_1.default.resolve(packageDir, 'tslint.json'), context);
        if (packageDir === path_1.default.resolve(process.cwd())) {
            yield utils_1.renderTemplate(baseTsConfigTemplate, path_1.default.resolve(packageDir, 'tsconfig.json'), context);
        }
        else {
            yield utils_1.renderTemplate(tsconfigTemplate, path_1.default.resolve(packageDir, 'tsconfig.json'), context);
        }
        const appTemplateDir = path_1.default.join(__dirname, 'src-template');
        for (const { path: p } of klaw_sync_1.default(appTemplateDir, { nodir: true })) {
            const contents = yield fs_extra_1.readFile(p);
            const relativePath = path_1.default.relative(appTemplateDir, p);
            const extension = path_1.default.extname(p);
            yield fs_extra_1.mkdirs(path_1.default.resolve(packageDir, 'src', path_1.default.dirname(relativePath)));
            yield fs_extra_1.writeFile(path_1.default.resolve(packageDir, 'src', relativePath), ['.ts', '.tsx'].includes(extension)
                ? utils_1.licenseHeader + contents.toString()
                : contents);
        }
    });
}
run()
    .then(() => {
    process.exit(0);
})
    .catch((err) => {
    console.log('Error running script.', err);
    process.exit(1);
});
//# sourceMappingURL=index.js.map