"use strict";
/*
 * MIT License
 *
 * Copyright (c) 2018 Choko (choko@curioswitch.org)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable:no-var-requires
process.env.NODE_ENV = 'production';
require('ts-node').register({
    ignore: [],
    compilerOptions: {
        module: 'commonjs',
        jsx: 'react',
        target: 'es2016',
    },
});
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const brotli_webpack_plugin_1 = __importDefault(require("brotli-webpack-plugin"));
const favicons_webpack_plugin_1 = __importDefault(require("favicons-webpack-plugin"));
const html_webpack_plugin_1 = __importDefault(require("html-webpack-plugin"));
const webpack_1 = require("react-loadable/webpack");
const static_site_generator_webpack_plugin_1 = __importDefault(require("static-site-generator-webpack-plugin"));
const webpack_2 = require("webpack");
const zopfli_webpack_plugin_1 = __importDefault(require("zopfli-webpack-plugin"));
const base_1 = __importDefault(require("./base"));
const prerenderConfigPath = path_1.default.resolve(process.cwd(), 'src/prerender');
const prerenderConfig = ['.ts', '.tsx', '.js', '.jsx']
    .map((ext) => `${prerenderConfigPath}${ext}`)
    .some((p) => fs_1.default.existsSync(p))
    ? require(prerenderConfigPath).default
    : undefined;
const loadableJsonPath = path_1.default.resolve(process.cwd(), 'build/react-loadable.json');
const plugins = [
    new webpack_2.DefinePlugin({
        'process.env': {
            APP_CONFIG_PATH: JSON.stringify(path_1.default.resolve(process.cwd(), 'src/app')),
            NODE_ENV: JSON.stringify('production'),
        },
        WEBPACK_PRERENDERING: false,
    }),
    new webpack_1.ReactLoadablePlugin({
        filename: loadableJsonPath,
    }),
    new html_webpack_plugin_1.default({
        inject: true,
        template: 'src/index.html',
        chunksSortMode: 'none',
    }),
    new favicons_webpack_plugin_1.default({
        inject: false,
        logo: 'favicon.png',
        prefix: 'icons-[hash]/',
        emitStats: true,
        statsFilename: 'iconstats.json',
    }),
    new zopfli_webpack_plugin_1.default({
        asset: '[path].gz[query]',
        algorithm: 'zopfli',
        test: /\.(js|css|html|svg)$/,
        threshold: 1024,
        minRatio: 0.9,
    }),
    new brotli_webpack_plugin_1.default({
        asset: '[path].br[query]',
        test: /\.(js|css|html|svg)$/,
        threshold: 1024,
        minRatio: 0.9,
    }),
];
exports.appConfiguration = base_1.default({
    plugins,
    mode: 'production',
    babelPlugins: [
        '@babel/transform-react-constant-elements',
        '@babel/transform-react-inline-elements',
        'react-loadable/babel',
    ],
    output: {
        filename: '[name].[chunkhash].js',
        chunkFilename: '[name].[chunkhash].chunk.js',
        publicPath: '/static/',
    },
});
function createPrerenderConfiguration() {
    const prerenderPlugins = [
        new webpack_2.DefinePlugin({
            'process.env': {
                APP_CONFIG_PATH: JSON.stringify(path_1.default.resolve(process.cwd(), 'src/app')),
                PRERENDER_CONFIG_PATH: JSON.stringify(path_1.default.resolve(process.cwd(), 'src/prerender')),
                NODE_ENV: JSON.stringify('production'),
                LOADABLE_JSON_PATH: JSON.stringify(loadableJsonPath),
                ICONSTATS_JSON_PATH: JSON.stringify(path_1.default.resolve(process.cwd(), 'build/web/iconstats.json')),
            },
            WEBPACK_PRERENDERING: true,
        }),
        new static_site_generator_webpack_plugin_1.default({
            entry: 'prerender',
            paths: Object.keys(prerenderConfig.paths),
            locals: {
                pathStates: prerenderConfig.paths,
            },
            globals: Object.assign({ window: {} }, prerenderConfig.globals),
        }),
        new zopfli_webpack_plugin_1.default({
            asset: '[path].gz[query]',
            algorithm: 'zopfli',
            test: /\.(html)$/,
            threshold: 1024,
            minRatio: 0.9,
        }),
        new brotli_webpack_plugin_1.default({
            asset: '[path].br[query]',
            test: /\.(html)$/,
            threshold: 1024,
            minRatio: 0.9,
        }),
    ];
    return base_1.default({
        plugins: prerenderPlugins,
        // This is only run during the build, so development mode is better to provide stacktraces.
        mode: 'development',
        target: 'node',
        entrypoints: {
            prerender: path_1.default.resolve(__dirname, '../../prerender/index.tsx'),
        },
        babelPlugins: [
            '@babel/transform-react-constant-elements',
            '@babel/transform-react-inline-elements',
            'react-loadable/babel',
            'dynamic-import-node',
        ],
        babelTargets: {
            node: 'current',
        },
        output: {
            filename: '../prerender/[name].js',
            chunkFilename: '../prerender/[name].chunk.js',
            publicPath: '/static/',
            libraryTarget: 'commonjs',
        },
    });
}
exports.prerenderConfiguration = prerenderConfig
    ? createPrerenderConfiguration()
    : undefined;
//# sourceMappingURL=prod.js.map